"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var fs = require('fs-extra');
var path = require('path');
var ora = require('ora');
var kebabCase = require('lodash.kebabcase');
var print = require('@toptal/davinci-cli-shared').print;
var workspace = require('../utils/workspace');
var skeleton = require('../utils/skeleton');
var _a = require('../constants'), LIBS_PATH = _a.LIBS_PATH, NAMESPACES_PATH = _a.NAMESPACES_PATH;
var getLibPath = function (libName, namespace) {
    return namespace
        ? path.join(NAMESPACES_PATH, namespace, LIBS_PATH, libName)
        : path.join(LIBS_PATH, libName);
};
var checkPrerequisites = function (monorepoWorkspaceDir, libName, namespace) { return __awaiter(void 0, void 0, void 0, function () {
    var spinner, isWorkspace, libPath, doesLibExist;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                spinner = ora('Checking prerequisites...').start();
                isWorkspace = Boolean(monorepoWorkspaceDir);
                if (!isWorkspace) {
                    spinner.fail();
                    throw new Error("It looks like you run 'new-lib' Davinci command not in a monorepo.");
                }
                libPath = path.join(monorepoWorkspaceDir, getLibPath(libName, namespace));
                return [4 /*yield*/, fs.exists(libPath)];
            case 1:
                doesLibExist = _a.sent();
                if (doesLibExist) {
                    spinner.fail();
                    throw new Error("Library already exists in the folder ".concat(libPath, "!"));
                }
                spinner.succeed('Prerequisites checked!');
                return [2 /*return*/];
        }
    });
}); };
var copyLibrarySkeleton = function (destDir, currentDavinciRunningDir) { return __awaiter(void 0, void 0, void 0, function () {
    var skeletonLibPath, appDir, spinner, err_1;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                skeletonLibPath = require.resolve('@toptal/davinci-skeleton-lib', {
                    paths: [currentDavinciRunningDir],
                });
                appDir = path.join(skeletonLibPath, '../..');
                spinner = ora('Copying skeleton...').start();
                _a.label = 1;
            case 1:
                _a.trys.push([1, 4, , 5]);
                return [4 /*yield*/, skeleton.copyAllFiles(appDir, destDir)];
            case 2:
                _a.sent();
                return [4 /*yield*/, skeleton.copyTemplatePackageJsonFile(destDir)];
            case 3:
                _a.sent();
                spinner.succeed('Skeleton copied!');
                return [3 /*break*/, 5];
            case 4:
                err_1 = _a.sent();
                spinner.fail();
                throw err_1;
            case 5: return [2 /*return*/];
        }
    });
}); };
var modifyPackageJson = function (destDir, libName, _a) {
    var namespace = _a.namespace, workspacePackageName = _a.workspacePackageName;
    return __awaiter(void 0, void 0, void 0, function () {
        var spinner, config;
        return __generator(this, function (_b) {
            spinner = ora('Modifying package.json...').start();
            try {
                config = skeleton.readPackageJson(destDir);
                config.name = namespace
                    ? "@toptal/".concat(namespace, "-").concat(libName)
                    : "@toptal/".concat(libName);
                config.homepage = "https://github.com/toptal/".concat(workspacePackageName, "/tree/master/").concat(getLibPath(libName, namespace));
                delete config.gitHead;
                delete config.publishConfig;
                config.license = 'SEE LICENSE IN LICENSE.MD';
                config.version = '0.0.1';
                skeleton.writePackageJson(destDir, config);
                spinner.succeed('package.json modified!');
            }
            catch (err) {
                spinner.fail();
                throw err;
            }
            return [2 /*return*/];
        });
    });
};
var installDependencies = function (monorepoWorkspaceDir) { return __awaiter(void 0, void 0, void 0, function () {
    var spinner, err_2;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                spinner = ora('Installing dependencies... ').start();
                _a.label = 1;
            case 1:
                _a.trys.push([1, 3, , 4]);
                return [4 /*yield*/, workspace.installDependencies(monorepoWorkspaceDir)];
            case 2:
                _a.sent();
                spinner.succeed('Dependencies installed!');
                return [3 /*break*/, 4];
            case 3:
                err_2 = _a.sent();
                spinner.fail();
                throw new Error(err_2);
            case 4: return [2 /*return*/];
        }
    });
}); };
var createCommand = function (libName, namespace) { return __awaiter(void 0, void 0, void 0, function () {
    var monorepoWorkspaceDir, workspacePackageJson, currentDavinciRunningDir, destDir, workspacePackageName, err_3;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                print.green('Creating', libName, namespace ? "in namespace ".concat(namespace) : '', '...');
                monorepoWorkspaceDir = workspace.getRootDir();
                workspacePackageJson = workspace.readPackageJson(monorepoWorkspaceDir);
                currentDavinciRunningDir = __dirname;
                _a.label = 1;
            case 1:
                _a.trys.push([1, 6, , 7]);
                // check if all apps are installed, directory doesn't exists, etc.
                return [4 /*yield*/, checkPrerequisites(monorepoWorkspaceDir, libName, namespace)];
            case 2:
                // check if all apps are installed, directory doesn't exists, etc.
                _a.sent();
                destDir = path.join(monorepoWorkspaceDir, getLibPath(libName, namespace));
                workspacePackageName = workspace.getPackageName(workspacePackageJson);
                // copy skeleton to destination directory
                return [4 /*yield*/, copyLibrarySkeleton(destDir, currentDavinciRunningDir)
                    // change package.json
                ];
            case 3:
                // copy skeleton to destination directory
                _a.sent();
                // change package.json
                return [4 /*yield*/, modifyPackageJson(destDir, libName, {
                        namespace: namespace,
                        workspacePackageName: workspacePackageName,
                    })
                    // run yarn
                ];
            case 4:
                // change package.json
                _a.sent();
                // run yarn
                return [4 /*yield*/, installDependencies(monorepoWorkspaceDir)];
            case 5:
                // run yarn
                _a.sent();
                print.green('Library has been created.');
                return [3 /*break*/, 7];
            case 6:
                err_3 = _a.sent();
                print.red(err_3);
                process.exit(1);
                return [3 /*break*/, 7];
            case 7: return [2 /*return*/];
        }
    });
}); };
var newLibCommandCreator = {
    command: 'new-lib <libName>',
    description: 'Create new library in monorepo',
    options: [
        {
            label: "Namespace of the lib. Creates the lib in a monorepo namespace: './namespaces/{namespace}/libs/{libName}'",
            name: '--namespace <namespace>',
        },
    ],
    action: function (_a) {
        var libName = _a.libName, namespace = _a.options.namespace;
        return __awaiter(void 0, void 0, void 0, function () {
            var formattedLibName;
            return __generator(this, function (_b) {
                formattedLibName = kebabCase(libName);
                return [2 /*return*/, createCommand(formattedLibName, namespace)];
            });
        });
    },
};
module.exports = newLibCommandCreator;
